# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from unittest import mock

from delfin.drivers import fake_storage
from delfin.task_manager.tasks import task
from delfin.task_manager.tasks.task import StorageDeviceTask

from delfin import test, context, coordination

storage = {
    'id': '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
    'name': 'fake_driver',
    'description': 'it is a fake driver.',
    'vendor': 'fake_vendor',
    'model': 'fake_model',
    'status': 'normal',
    'serial_number': '2102453JPN12KA000011',
    'firmware_version': '1.0.0',
    'location': 'HK',
    'total_capacity': 1024 * 1024,
    'used_capacity': 3126,
    'free_capacity': 1045449,
}

pools_list = [{
    'id': '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
    "name": "fake_pool_" + str(id),
    "storage_id": '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
    "native_storage_pool_id": "fake_original_id_" + str(id),
    "description": "Fake Pool",
    "status": "normal",
    "total_capacity": 1024 * 1024,
    "used_capacity": 3126,
    "free_capacity": 1045449,
}
]

vols_list = [{
    'id': '12c2d52f-01bc-41f5-b73f-7abf6f38a340',
    "name": "fake_vol_" + str(id),
    "storage_id": '12c2d52f-01bc-41f5-b73f-7abf6f38a2a6',
    "description": "Fake Volume",
    "status": "normal",
    "native_volume_id": "fake_original_id_" + str(id),
    "wwn": "fake_wwn_" + str(id),
    "total_capacity": 1024 * 1024,
    "used_capacity": 3126,
    "free_capacity": 1045449,
}
]


class TestStorageDeviceTask(test.TestCase):
    def setUp(self):
        super(TestStorageDeviceTask, self).setUp()
        self.driver_api = mock.Mock()
        self.task_manager = StorageDeviceTask(
            context, "12c2d52f-01bc-41f5-b73f-7abf6f38a2a6")
        self.mock_object(self.task_manager, 'driver_api', self.driver_api)

    @mock.patch.object(coordination.LOCK_COORDINATOR, 'get_lock')
    @mock.patch('delfin.drivers.api.API.get_storage')
    @mock.patch('delfin.db.storage_update')
    @mock.patch('delfin.db.storage_get')
    @mock.patch('delfin.db.storage_delete')
    @mock.patch('delfin.db.access_info_delete')
    @mock.patch('delfin.db.alert_source_delete')
    def test_sync_successful(self, alert_source_delete, access_info_delete,
                             mock_storage_delete, mock_storage_get,
                             mock_storage_update, mock_get_storage, get_lock):
        storage_obj = task.StorageDeviceTask(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')

        storage_obj.sync()
        self.assertTrue(get_lock.called)
        self.assertTrue(mock_storage_get.called)
        self.assertTrue(mock_storage_delete.called)
        self.assertTrue(access_info_delete.called)
        self.assertTrue(alert_source_delete.called)
        self.assertTrue(mock_storage_update.called)
        mock_get_storage.assert_called_with(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')

        fake_storage_obj = fake_storage.FakeStorageDriver()
        mock_get_storage.return_value = fake_storage_obj.get_storage(context)
        storage_obj.sync()

    @mock.patch('delfin.db.storage_delete')
    @mock.patch('delfin.db.alert_source_delete')
    def test_successful_remove(self, mock_alert_del, mock_strg_del):
        storage_obj = task.StorageDeviceTask(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')
        storage_obj.remove()

        mock_strg_del.assert_called_with(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')
        mock_alert_del.assert_called_with(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')


class TestStoragePoolTask(test.TestCase):
    @mock.patch.object(coordination.LOCK_COORDINATOR, 'get_lock')
    @mock.patch('delfin.drivers.api.API.list_storage_pools')
    @mock.patch('delfin.db.storage_pool_get_all')
    @mock.patch('delfin.db.storage_pools_delete')
    @mock.patch('delfin.db.storage_pools_update')
    @mock.patch('delfin.db.storage_pools_create')
    def test_sync_successful(self, mock_pool_create, mock_pool_update,
                             mock_pool_del, mock_pool_get_all,
                             mock_list_pools, get_lock):
        pool_obj = task.StoragePoolTask(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')
        pool_obj.sync()

        self.assertTrue(mock_list_pools.called)
        self.assertTrue(mock_pool_get_all.called)
        self.assertTrue(get_lock.called)

        # collect the pools from fake_storage
        fake_storage_obj = fake_storage.FakeStorageDriver()

        # add the new pool to DB
        mock_list_pools.return_value = fake_storage_obj.list_storage_pools(
            context)
        mock_pool_get_all.return_value = list()
        pool_obj.sync()
        self.assertTrue(mock_pool_create.called)

        # update the new pool of DB
        mock_list_pools.return_value = pools_list
        mock_pool_get_all.return_value = pools_list
        pool_obj.sync()
        self.assertTrue(mock_pool_update.called)

        # delete the new pool to DB
        mock_list_pools.return_value = list()
        mock_pool_get_all.return_value = pools_list
        pool_obj.sync()
        self.assertTrue(mock_pool_del.called)

    @mock.patch('delfin.db.storage_pool_delete_by_storage')
    def test_remove(self, mock_pool_del):
        pool_obj = task.StoragePoolTask(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')
        pool_obj.remove()
        self.assertTrue(mock_pool_del.called)


class TestStorageVolumeTask(test.TestCase):
    @mock.patch.object(coordination.LOCK_COORDINATOR, 'get_lock')
    @mock.patch('delfin.drivers.api.API.list_volumes')
    @mock.patch('delfin.db.volume_get_all')
    @mock.patch('delfin.db.volumes_delete')
    @mock.patch('delfin.db.volumes_update')
    @mock.patch('delfin.db.volumes_create')
    def test_sync_successful(self, mock_vol_create, mock_vol_update,
                             mock_vol_del, mock_vol_get_all, mock_list_vols,
                             get_lock):
        vol_obj = task.StorageVolumeTask(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')
        vol_obj.sync()
        self.assertTrue(mock_list_vols.called)
        self.assertTrue(mock_vol_get_all.called)
        self.assertTrue(get_lock.called)

        # collect the volumes from fake_storage
        fake_storage_obj = fake_storage.FakeStorageDriver()

        # add the volumes to DB
        mock_list_vols.return_value = fake_storage_obj.list_volumes(context)
        mock_vol_get_all.return_value = list()
        vol_obj.sync()
        self.assertTrue(mock_vol_create.called)

        # update the volumes to DB
        mock_list_vols.return_value = vols_list
        mock_vol_get_all.return_value = vols_list
        vol_obj.sync()
        self.assertTrue(mock_vol_update.called)

        # delete the volumes to DB
        mock_list_vols.return_value = list()
        mock_vol_get_all.return_value = vols_list
        vol_obj.sync()
        self.assertTrue(mock_vol_del.called)

    @mock.patch('delfin.db.volume_delete_by_storage')
    def test_remove(self, mock_vol_del):
        vol_obj = task.StorageVolumeTask(
            context, 'c5c91c98-91aa-40e6-85ac-37a1d3b32bda')
        vol_obj.remove()
        self.assertTrue(mock_vol_del.called)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from oslo_utils import importutils

from delfin import exception
from delfin.common import constants


class AlertHandlerTestCase(unittest.TestCase):
    ALERT_HANDLER_CLASS = 'delfin.drivers.dell_emc.vmax.alert_handler' \
                          '.snmp_alerts.AlertHandler'

    def _get_alert_handler(self):
        alert_handler_class = importutils.import_class(
            self.ALERT_HANDLER_CLASS)
        alert_handler = alert_handler_class()
        return alert_handler

    def _get_fake_alert_info(self):
        alert_info = {
            '1.3.6.1.3.94.1.11.1.3.0': 79,
            '1.3.6.1.3.94.1.6.1.20.0': '000192601409',
            '1.3.6.1.3.94.1.11.1.7.0': 'topology',
            '1.3.6.1.3.94.1.11.1.9.0': 'Symmetrix 000192601409 FastSRP '
                                       'SRP_1 : Remote (SRDF) diagnostic '
                                       'event trace triggered.',
            '1.3.6.1.3.94.1.11.1.6.0': '6',
            '1.3.6.1.3.94.1.6.1.3.0': 'storage-subsystem',
            '1.3.6.1.4.1.1139.3.8888.1.0.0': 'symmetrix',
            '1.3.6.1.4.1.1139.3.8888.2.0.0': '1050',
            '1.3.6.1.4.1.1139.3.8888.3.0.0': '1051',
            '1.3.6.1.4.1.1139.3.8888.4.0.0': 'SRP_1'}

        return alert_info

    def test_parse_alert_with_all_necessary_info(self):
        """ Success flow with all necessary parameters"""
        alert_handler_inst = self._get_alert_handler()
        alert = self._get_fake_alert_info()

        expected_alert_model = {
            'alert_id': alert['1.3.6.1.4.1.1139.3.8888.2.0.0'],
            'alert_name': 'SYMAPI_AEVENT2_UID_MOD_DIAG_TRACE_TRIG',
            'severity': constants.Severity.WARNING,
            'category': constants.Category.NOT_SPECIFIED,
            'type': constants.EventType.EQUIPMENT_ALARM,
            'sequence_number': alert['1.3.6.1.3.94.1.11.1.3.0'],
            'description': alert['1.3.6.1.3.94.1.11.1.9.0'],
            'recovery_advice': 'None',
            'resource_type': alert['1.3.6.1.3.94.1.6.1.3.0'],
            'location': 'Array id=000192601409,'
                        'Component type=Symmetrix Disk '
                        'Group,'
                        'Component name=SRP_1,'
                        'Event source=symmetrix',
        }
        context = {}
        alert_model = alert_handler_inst.parse_alert(context, alert)

        # occur_time depends on current time
        # Verify that all other fields are matching
        expected_alert_model['occur_time'] = alert_model['occur_time']
        self.assertDictEqual(expected_alert_model, alert_model)

    def test_parse_alert_without_mandatory_info(self):
        """ Error flow with some mandatory parameters missing"""
        alert_handler_inst = self._get_alert_handler()
        context = {}
        alert = self._get_fake_alert_info()
        alert['1.3.6.1.3.94.1.11.1.6.0'] = ''
        self.assertRaisesRegex(exception.InvalidInput, "Mandatory information "
                                                       "connUnitEventSeverity"
                                                       " missing",
                               alert_handler_inst.parse_alert, context, alert)



# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages


setup(
    name="delfin",
    version="0.4.0",
    author="SODA Authors",
    author_email="Opensds-tech-discuss@lists.opensds.io",
    license="Apache 2.0",
    packages=find_packages(exclude=("tests", "tests.*")),
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*",
    entry_points={
        'delfin.alert.exporters': [
            'example = delfin.exporter.example:AlertExporterExample'
        ],
        'delfin.performance.exporters': [
            'example = delfin.exporter.example:PerformanceExporterExample'
        ],
        'delfin.storage.drivers': [
            'fake_storage fake_driver = delfin.drivers.fake_storage:FakeStorageDriver',
            'dellemc vmax = delfin.drivers.dell_emc.vmax.vmax:VMAXStorageDriver',
            'hpe 3par = delfin.drivers.hpe.hpe_3par.hpe_3parstor:Hpe3parStorDriver',
            'huawei oceanstor = delfin.drivers.huawei.oceanstor.oceanstor:OceanStorDriver'
        ]
    },
)



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# This file contains mapping of alert information to vmax specific description

# component type to descriptive info
component_type_mapping = {
    '1024': 'Symmetrix',
    '1025': 'Service Processor',
    '1026': 'Device',
    '1027': 'Physical Disk',
    '1028': 'Director',
    '1029': 'Port',
    '1030': 'SRDF sub-system',
    '1031': 'SRDF group',
    '1032': 'Snap Save Device Pool',
    '1033': 'Cache / Memory',
    '1034': 'Power or Battery subsystem',
    '1035': 'Environmental (e.g.: Temperature, Smoke)',
    '1036': 'Diagnostics',
    '1037': 'Communications sub-system',
    '1038': 'External Lock',
    '1039': 'Fan',
    '1040': 'Link Controller Card',
    '1041': 'Enclosure, Enclosure-Slot or MIBE',
    '1042': 'SRDF/A DSE Device Pool',
    '1043': 'Thin Device Data Pool',
    '1044': 'Solutions Enabler DG group',
    '1045': 'Solutions Enabler CG group',
    '1046': 'Management Module',
    '1047': 'IO Module Carrier',
    '1048': 'Director - Environmental',
    '1049': 'Storage Group',
    '1050': 'Migration Session',
    '1051': 'Symmetrix Disk Group'
}

# Alarm id to alarm name mapping
# Currently this contains limited list, to be extended
alarm_id_name_mapping = {
    '1': 'SYMAPI_AEVENT2_UID_EVT_RESTARTED',
    '2': 'SYMAPI_AEVENT2_UID_EVT_EVENTS_LOST',
    '3': 'SYMAPI_AEVENT2_UID_EVT_EVENTS_OVERFLOW',
    '1050': 'SYMAPI_AEVENT2_UID_MOD_DIAG_TRACE_TRIG',
    '1051': 'SYMAPI_AEVENT2_UID_MOD_DIAG_TRACE_TRIG_REM'
}



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_log import log

from delfin import exception
from delfin.common import constants

LOG = log.getLogger(__name__)


class AlertHandler(object):
    """Alert handling functions for unisphere alerts"""

    # Alert Id and name are not part of queried alerts
    DEFAULT_UNISPHERE_ALERT_NAME = "Unisphere alert about vmax"
    DEFAULT_UNISPHERE_ALERT_ID = 0xFFFFFFFF

    # Translation of queried alert severity to alert model severity
    SEVERITY_MAP = {"FATAL": constants.Severity.FATAL,
                    "CRITICAL": constants.Severity.CRITICAL,
                    "WARNING": constants.Severity.WARNING,
                    "NORMAL": constants.Severity.INFORMATIONAL,
                    "INFORMATION": constants.Severity.INFORMATIONAL}

    def __init__(self):
        pass

    def parse_queried_alerts(self, alert_list):
        """Parse queried alerts and convert to alert model."""

        alert_model_list = []
        for alert in alert_list:
            try:
                alert_model = dict()
                alert_model['alert_id'] = self.DEFAULT_UNISPHERE_ALERT_ID
                alert_model['alert_name'] = self.DEFAULT_UNISPHERE_ALERT_NAME

                alert_model['severity'] = self.SEVERITY_MAP.get(
                    alert['severity'], constants.Severity.NOT_SPECIFIED)

                # category and type are not part of queried alerts
                alert_model['category'] = constants.Category.EVENT
                alert_model['type'] = constants.EventType.EQUIPMENT_ALARM

                alert_model['sequence_number'] = alert['alertId']
                alert_model['occur_time'] = alert['created_date_milliseconds']
                alert_model['description'] = alert['description']
                alert_model['recovery_advice'] = 'None'
                alert_model['resource_type'] = constants.DEFAULT_RESOURCE_TYPE

                # Location is name-value pair
                alert_model['location'] = 'type=' + alert['type']
                alert_model_list.append(alert_model)
            except Exception as e:
                LOG.error(e)
                msg = ("Failed to build alert model as some attributes "
                       "missing in alert message")
                raise exception.InvalidResults(msg)
        return alert_model_list

    def add_trap_config(self, context, storage_id, trap_config):
        """Config the trap receiver in storage system."""
        pass

    def remove_trap_config(self, context, storage_id, trap_config):
        """Remove trap receiver configuration from storage system."""
        pass

    def clear_alert(self, context, storage_id, alert):
        """Clear alert from storage system."""
        pass



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import time

from oslo_log import log as logging

from delfin import exception
from delfin.common import constants
from delfin.drivers.hpe.hpe_3par import consts
from delfin.i18n import _

LOG = logging.getLogger(__name__)


class AlertHandler(object):
    """Alert handling functions for Hpe3 parstor driver"""
    # messageCode
    OID_MESSAGECODE = '1.3.6.1.4.1.12925.1.7.1.8.1'
    # severity
    OID_SEVERITY = '1.3.6.1.4.1.12925.1.7.1.2.1'
    # state
    OID_STATE = '1.3.6.1.4.1.12925.1.7.1.9.1'
    # id
    OID_ID = '1.3.6.1.4.1.12925.1.7.1.7.1'
    # timeOccurred
    OID_TIMEOCCURRED = '1.3.6.1.4.1.12925.1.7.1.3.1'
    # details
    OID_DETAILS = '1.3.6.1.4.1.12925.1.7.1.6.1'
    # component
    OID_COMPONENT = '1.3.6.1.4.1.12925.1.7.1.5.1'

    # Translation of trap severity to alert model severity
    SEVERITY_MAP = {"1": constants.Severity.CRITICAL,
                    "2": constants.Severity.MAJOR,
                    "3": constants.Severity.MINOR,
                    "4": constants.Severity.WARNING,
                    "0": constants.Severity.FATAL,
                    "5": constants.Severity.INFORMATIONAL,
                    "6": constants.Severity.NOT_SPECIFIED}

    # Translation of trap alert category to alert model category
    CATEGORY_MAP = {"0": constants.Category.NOT_SPECIFIED,
                    "1": constants.Category.FAULT,
                    "2": constants.Category.RECOVERY,
                    "3": constants.Category.RECOVERY,
                    "4": constants.Category.RECOVERY,
                    "5": constants.Category.RECOVERY}

    # Attributes expected in alert info to proceed with model filling
    _mandatory_alert_attributes = (
        OID_MESSAGECODE,
        OID_SEVERITY,
        OID_STATE,
        OID_ID,
        OID_TIMEOCCURRED,
        OID_DETAILS,
        OID_COMPONENT
    )

    # Convert received time to epoch format
    TIME_PATTERN = '%Y-%m-%d %H:%M:%S CST'

    default_me_category = 'storage-subsystem'

    def __init__(self, resthanlder=None, sshhanlder=None):
        self.resthanlder = resthanlder
        self.sshhanlder = sshhanlder

    def parse_alert(self, context, alert):
        """Parse alert data got from alert manager and fill the alert model."""
        # Check for mandatory alert attributes
        for attr in self._mandatory_alert_attributes:
            if not alert.get(attr):
                msg = "Mandatory information %s missing in alert message. " \
                      % attr
                raise exception.InvalidInput(msg)

        try:
            alert_model = dict()
            # These information are sourced from device registration info
            alert_model['alert_id'] = alert.get(AlertHandler.OID_MESSAGECODE)
            alert_model['alert_name'] = self.get_alert_type(alert.get(
                AlertHandler.OID_MESSAGECODE))
            alert_model['severity'] = self.SEVERITY_MAP.get(
                alert.get(AlertHandler.OID_SEVERITY),
                constants.Severity.NOT_SPECIFIED)
            alert_model['category'] = self.CATEGORY_MAP.get(
                alert.get(AlertHandler.OID_STATE),
                constants.Category.NOT_SPECIFIED)
            alert_model['type'] = constants.EventType.EQUIPMENT_ALARM
            alert_model['sequence_number'] = alert.get(AlertHandler.OID_ID)
            alert_model['occur_time'] = self.get_time_stamp(
                alert.get(AlertHandler.OID_TIMEOCCURRED))
            alert_model['description'] = alert.get(AlertHandler.OID_DETAILS)
            alert_model['resource_type'] = constants.DEFAULT_RESOURCE_TYPE
            alert_model['location'] = alert.get(AlertHandler.OID_COMPONENT)

            if alert.get(AlertHandler.OID_STATE) == '5':
                alert_model['clear_category'] = constants.ClearType.AUTOMATIC
            return alert_model
        except Exception as e:
            LOG.error(e)
            msg = (_("Failed to build alert model as some attributes missing "
                     "in alert message."))
            raise exception.InvalidResults(msg)

    def add_trap_config(self, context, storage_id, trap_config):
        """Config the trap receiver in storage system."""
        # Currently not implemented
        pass

    def remove_trap_config(self, context, storage_id, trap_config):
        """Remove trap receiver configuration from storage system."""
        # Currently not implemented
        pass

    def clear_alert(self, context, alert):
        """Clear alert from storage system.
            Currently not implemented   removes command : removealert
        """
        re = 'Failed'
        try:
            if alert is not None:
                # alert is sequence number here
                re = self.sshhanlder.remove_alerts(context, alert)
                if not re:
                    re = 'Success'
                else:
                    LOG.error('remove failed:{}'.format(re))
                    raise exception.SSHException(re)
        except Exception as e:
            LOG.error(e)
            raise exception.SSHException(
                reason='Failed to ssh Hpe3parStor')
        return re

    def list_alerts(self, context):
        try:
            # Get list of Hpe3parStor alerts
            alert_list = []
            try:
                reslist = self.sshhanlder.get_all_alerts(context)
            except Exception as e:
                LOG.error(e)
                raise exception.SSHException(
                    reason='Failed to ssh Hpe3parStor')
            message_code = ''
            event_type = ''
            severity = ''
            state = ''
            alarm_id = ''
            occur_time = ''
            alert_message = ''
            component = ''

            alertlist = reslist.split('\n')

            for alertinfo in alertlist:
                strline = alertinfo
                if strline is not None and strline != '':
                    # strline = strline.replace(" ", "")
                    strinfo = strline.split(': ', 1)
                    strinfo[0] = strinfo[0].replace(" ", "")
                    if strinfo[0] == 'Id':
                        alarm_id = strinfo[1]
                    elif strinfo[0] == 'State':
                        if strinfo[1] == 'New':
                            state = constants.Category.FAULT
                    elif strinfo[0] == 'MessageCode':
                        message_code = strinfo[1]
                    elif strinfo[0] == 'Time':
                        occur_time = strinfo[1]
                    elif strinfo[0] == 'Severity':
                        if strinfo[1] == 'Major':
                            severity = constants.Severity.MAJOR
                        elif strinfo[1] == 'Minor':
                            severity = constants.Severity.MINOR
                        elif strinfo[1] == 'Critical':
                            severity = constants.Severity.CRITICAL
                        elif strinfo[1] == 'Degraded':
                            severity = constants.Severity.WARNING
                        elif strinfo[1] == 'Fatal':
                            severity = constants.Severity.FATAL
                        elif strinfo[1] == 'Informational':
                            severity = constants.Severity.INFORMATIONAL
                        elif strinfo[1] == 'Debug':
                            severity = constants.Severity.NOT_SPECIFIED
                    elif strinfo[0] == 'Type':
                        event_type = strinfo[1]
                    elif strinfo[0] == 'Message':
                        alert_message = strinfo[1]
                    elif strinfo[0] == 'Component':
                        component = strinfo[1]

                        alert_model = {
                            'alert_id': str(int(message_code, 16)),
                            'alert_name': event_type,
                            'severity': severity,
                            'category': state,
                            'type': constants.EventType.EQUIPMENT_ALARM,
                            'sequence_number': alarm_id,
                            'occur_time': self.get_time_stamp(occur_time),
                            'description': alert_message,
                            'resource_type': constants.DEFAULT_RESOURCE_TYPE,
                            'location': component
                        }

                        alert_list.append(alert_model)
                        message_code = ''
                        event_type = ''
                        severity = ''
                        state = ''
                        alarm_id = ''
                        occur_time = ''
                        alert_message = ''
                        component = ''
            return alert_list

        except Exception as err:
            LOG.error(
                "Failed to get pool metrics from Hpe3parStor: {}".format(err))
            raise exception.StorageBackendException(
                reason='Failed to get pool metrics from Hpe3parStor')

    def get_time_stamp(self, time_str):
        """ Time stamp to time conversion
        """
        time_stamp = ''
        try:
            if time_str is not None:
                # Convert to time array first
                time_array = time.strptime(time_str, AlertHandler.TIME_PATTERN)
                # Convert to timestamps to milliseconds
                time_stamp = int(time.mktime(time_array) * 1000)
        except Exception as e:
            LOG.error(e)

        return time_stamp

    def get_alert_type(self, message_code):
        """
        Get alert type

        :param str message_code: alert's message_code.
        :return: returns alert's type
        """
        re = ''
        try:
            if message_code is not None:
                messagekey = \
                    (hex(int(message_code))).replace('0x', '0x0')
                re = consts.HPE3PAR_ALERT_CODE.get(messagekey)
        except Exception as e:
            LOG.error(e)

        return re



# Copyright 2020 The SODA Authors.
# Copyright (c) 2016 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

STATUS_HEALTH = '1'
STATUS_ACTIVE = '43'
STATUS_RUNNING = '10'
STATUS_VOLUME_READY = '27'
STATUS_LUNCOPY_READY = '40'
STATUS_QOS_ACTIVE = '2'
QOS_INACTIVATED = '45'
LUN_TYPE = '11'
SNAPSHOT_TYPE = '27'
STATUS_POOL_ONLINE = '27'
STATUS_STORAGE_NORMAL = '1'

ERROR_CONNECT_TO_SERVER = -403
ERROR_UNAUTHORIZED_TO_SERVER = -401

SOCKET_TIMEOUT = 52
LOGIN_SOCKET_TIMEOUT = 4

ERROR_VOLUME_NOT_EXIST = 1077939726
RELOGIN_ERROR_PASS = [ERROR_VOLUME_NOT_EXIST]
PWD_EXPIRED = 3
PWD_RESET = 4

BLOCK_STORAGE_POOL_TYPE = '1'
FILE_SYSTEM_POOL_TYPE = '2'

SECTORS_SIZE = 512
QUERY_PAGE_SIZE = 150

THICK_LUNTYPE = '0'
THIN_LUNTYPE = '1'



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from delfin.api import common
from delfin.api import extensions
from delfin.api.v1 import access_info
from delfin.api.v1 import alert_source
from delfin.api.v1 import alerts
from delfin.api.v1 import storage_pools
from delfin.api.v1 import storages
from delfin.api.v1 import volumes


class APIRouter(common.APIRouter):

    ExtensionManager = extensions.ExtensionManager

    def _setup_routes(self, mapper):
        mapper.redirect("", "/")

        self.resources['storages'] = storages.create_resource()
        mapper.resource("storage", "storages",
                        controller=self.resources['storages'],
                        member={'sync': 'POST'})

        mapper.connect("storages", "/storages/sync",
                       controller=self.resources['storages'],
                       action="sync_all",
                       conditions={"method": ["POST"]})

        self.resources['access_info'] = access_info.create_resource()
        mapper.connect("storages", "/storages/{id}/access-info",
                       controller=self.resources['access_info'],
                       action="show",
                       conditions={"method": ["GET"]})

        mapper.connect("storages", "/storages/{id}/access-info",
                       controller=self.resources['access_info'],
                       action="update",
                       conditions={"method": ["PUT"]})

        self.resources['alert_sources'] = alert_source.create_resource()
        mapper.connect("storages", "/storages/{id}/alert-source",
                       controller=self.resources['alert_sources'],
                       action="put",
                       conditions={"method": ["PUT"]})
        mapper.connect("storages", "/storages/{id}/alert-source",
                       controller=self.resources['alert_sources'],
                       action="show",
                       conditions={"method": ["GET"]})
        mapper.connect("storages", "/storages/{id}/alert-source",
                       controller=self.resources['alert_sources'],
                       action="delete",
                       conditions={"method": ["DELETE"]})

        self.resources['alerts'] = alerts.create_resource()
        mapper.connect("storages", "/storages/{id}/alerts/{sequence_number}",
                       controller=self.resources['alerts'],
                       action="delete",
                       conditions={"method": ["DELETE"]})

        mapper.connect("storages", "/storages/{id}/alerts",
                       controller=self.resources['alerts'],
                       action="show",
                       conditions={"method": ["GET"]})

        self.resources['storage-pools'] = storage_pools.create_resource()
        mapper.resource("storage-pool", "storage-pools",
                        controller=self.resources['storage-pools'])

        self.resources['volumes'] = volumes.create_resource()
        mapper.resource("volume", "volumes",
                        controller=self.resources['volumes'])



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http:#www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import time
from time import gmtime, strftime

from oslo_log import log

from delfin import exception
from delfin.common import constants
from delfin.drivers.dell_emc.vmax.alert_handler import alert_mapper
from delfin.drivers.dell_emc.vmax.alert_handler import oid_mapper

LOG = log.getLogger(__name__)


class AlertHandler(object):
    """Alert handling functions for vmax snmp traps"""

    def __init__(self):
        self.oid_mapper = oid_mapper.OidMapper()

    # Translation of trap severity to alert model severity
    # Values are: unknown=1, emergency=2, alert=3, critical=4, error=5,
    #             warning=6, alert=3, notify=7, info=8, debug=9, mark=10
    SEVERITY_MAP = {"2": constants.Severity.FATAL,
                    "3": constants.Severity.CRITICAL,
                    "4": constants.Severity.CRITICAL,
                    "5": constants.Severity.MAJOR,
                    "6": constants.Severity.WARNING,
                    "7": constants.Severity.WARNING,
                    "8": constants.Severity.INFORMATIONAL,
                    "9": constants.Severity.INFORMATIONAL,
                    "10": constants.Severity.INFORMATIONAL}

    # Attributes mandatory in alert info to proceed with model filling
    _mandatory_alert_attributes = ('emcAsyncEventCode',
                                   'connUnitEventSeverity',
                                   'connUnitEventType', 'connUnitEventDescr',
                                   'connUnitType',
                                   'emcAsyncEventComponentType',
                                   'emcAsyncEventComponentName',
                                   'emcAsyncEventSource')

    def parse_alert(self, context, alert):
        """Parse alert data got from alert manager and fill the alert model."""

        alert = self.oid_mapper.map_oids(alert)
        # Check for mandatory alert attributes
        for attr in self._mandatory_alert_attributes:
            if not alert.get(attr):
                msg = "Mandatory information %s missing in alert message. " \
                      % attr
                raise exception.InvalidInput(msg)

        alert_model = {}

        # Fill alarm id and fill alarm_name with corresponding mapping names
        alert_model['alert_id'] = alert['emcAsyncEventCode']
        alert_model['alert_name'] = alert_mapper.alarm_id_name_mapping.get(
            alert_model['alert_id'], alert_model['alert_id'])

        alert_model['severity'] = self.SEVERITY_MAP.get(
            alert['connUnitEventSeverity'],
            constants.Severity.INFORMATIONAL)

        alert_model['category'] = constants.Category.NOT_SPECIFIED
        alert_model['type'] = constants.EventType.EQUIPMENT_ALARM

        alert_model['sequence_number'] = alert['connUnitEventId']

        # trap info do not contain occur time, update with received time
        # Get date and time and convert to epoch format
        pattern = '%Y-%m-%d %H:%M:%S'
        curr_time = strftime(pattern, gmtime())

        alert_model['occur_time'] = int(time.mktime(time.strptime(curr_time,
                                                                  pattern)))
        alert_model['description'] = alert['connUnitEventDescr']
        alert_model['recovery_advice'] = 'None'
        alert_model['resource_type'] = alert['connUnitType']

        # Location is name-value pair having component type and component name
        component_type = alert_mapper.component_type_mapping.get(
            alert.get('emcAsyncEventComponentType'), "")
        alert_model['location'] = 'Array id=' \
                                  + alert['connUnitName'] \
                                  + ',Component type=' \
                                  + component_type \
                                  + ',Component name=' \
                                  + alert['emcAsyncEventComponentName'] \
                                  + ',Event source=' \
                                  + alert['emcAsyncEventSource']

        return alert_model

    def add_trap_config(self, context, storage_id, trap_config):
        """Config the trap receiver in storage system."""
        pass

    def remove_trap_config(self, context, storage_id, trap_config):
        """Remove trap receiver configuration from storage system."""
        pass

    def clear_alert(self, context, storage_id, alert):
        """Clear alert from storage system."""
        pass



# Copyright 2020 The SODA Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
from abc import ABCMeta, abstractmethod

from oslo_config import cfg
from oslo_utils import importutils

CONF = cfg.CONF


class ICryptor(metaclass=ABCMeta):

    @staticmethod
    @abstractmethod
    def encode(plain_text):
        pass

    @staticmethod
    @abstractmethod
    def decode(cipher_text):
        pass


class _Base64(ICryptor):

    @staticmethod
    def encode(data):
        """Base64 encode

        :param data: The plain text that need to be encode
        :type str:
        :return cipher data: The encoded cipher text
        :type str:
        """
        return base64.b64encode(data.encode()).decode('utf-8')

    @staticmethod
    def decode(data):
        """Base64 decode

        :param data: The cipher text that need to be decode
        :type str:
        :return plain data: The decoded plain text
        :type str:
        """
        return base64.b64decode(data).decode('utf-8')


_cryptor = importutils.import_class(CONF.delfin_cryptor)


def encode(plain_text):
    return _cryptor.encode(plain_text)


def decode(cipher_text):
    return _cryptor.decode(cipher_text)
